#!/usr/bin/perl

use strict;
use Test::More;
use FDN::Common::Config;
use Data::Dumper;

$FDN::Common::Config::CONFIG_ROOT = 't/';

my $nbtest;

BEGIN { $nbtest += 1; }

$FDN::Web::Session::debug=1;
my $sess = new_session('test');
ok ( defined $sess, 'new');
is ( ref($sess), 'HASH', 'new');
is ( $sess->{type}, 'test', 'new');
like ( $sess->{name}, qr/^test/, 'new');
BEGIN { $nbtest += 4; }

my $name = $sess->{name};
my $type = $sess->{type};
my $res;
$sess->{content}->{a} = 1;
$res = store_session($sess);
is ( $res, $name, 'store' );
BEGIN { $nbtest += 1; }

$sess = retrieve_session($type, $name);
ok ( defined $sess, 'retrieve');
is ( ref($sess), 'HASH', 'retrieve');
ok ( defined $sess->{type}, 'retrieve');
is ( $sess->{type}, $type, 'retrieve');
ok ( defined $sess->{name}, 'retrieve');
is ( $sess->{name}, $name, 'retrieve');
ok ( defined $sess->{content}, 'retrieve');
is ( ref $sess->{content}, 'HASH', 'retrieve' );
ok ( defined $sess->{content}->{a}, 'retrieve' );
is ( $sess->{content}->{a}, 1, 'retrieve' );
BEGIN { $nbtest += 10; }

sleep(1);

ok ( refresh_session($type, $name), 'refresh' );
BEGIN { $nbtest += 1; }

sleep(1);

ok ( refresh_session($type, $name), 'refresh' );
BEGIN { $nbtest += 1; }

sleep(1);
$sess = retrieve_session($type, $name);
ok ( defined $sess, 'refresh' );
BEGIN { $nbtest += 1; }

sleep(3);
$sess = retrieve_session($type, $name);
ok ( ! defined $sess, 'refresh' );
BEGIN { $nbtest += 1; }

$sess = new_session('test');
$name = $sess->{name};
$type = $sess->{type};
$sess->{content}->{a} = 1;
$res = store_session($sess);

is ( $res, $name, 'store' );
ok ( close_session($type, $name), 'close' );
$sess = retrieve_session($type, $name);
ok ( ! defined $sess, 'close' );
BEGIN { $nbtest += 3; }



$sess = new_session('test');
$name = $sess->{'name'};
$type = $sess->{'type'};

if ( ! fork() ) {
	# Je suis dans le fils
	# - J'attend une seconde
	# - Je note le temps
	# - Je demande le verrou (la session est bloque, j'attend 4 secondes)
	# - Je compare le temps, min 2 sec d'cart
	sleep(1);
	my $tmp = time();
	$sess = retrieve_session($type, $name, 1);
	my $delta = time() - $tmp;
	ok ( $delta > 2, 'lock on retrieve 1' );
	ok ( exists $sess->{'content'}->{'papa'}, 'lock on retrieve 2' );
	is ( $sess->{'content'}->{'papa'}, 'plop', 'lock on retrieve 3' );
	sleep(2);
	$sess->{'content'}->{'fils'} = 'coin';
	store_session($sess);
	exit(0);
} else {
	# Mon fils est suppos faire tourner ses 3 tests avant moi.
	# Ils sont pris de bien se passer... en tous cas, vu par moi, c'est
	# bon (c'est "prove" qui analysera rellement la sortie).
	my $tb = Test::More->builder;
	$tb->{'Curr_Test'} += 3;
	push @{$tb->{'Test_Results'}}, {'ok' => 1, name => 'skip', 'actual_ok' => 1, 'reason' => '', 'type' => ''};
	push @{$tb->{'Test_Results'}}, {'ok' => 1, name => 'skip', 'actual_ok' => 1, 'reason' => '', 'type' => ''};
	push @{$tb->{'Test_Results'}}, {'ok' => 1, name => 'skip', 'actual_ok' => 1, 'reason' => '', 'type' => ''};
}
BEGIN { $nbtest += 3; }

$sess = retrieve_session($type, $name, 1);
# Je tiens le verrou
sleep(4);
$sess->{'content'}->{'papa'} = 'plop';
store_session($sess);
# Le verrou saute, mon fils reprend la main

my $tmp = time();
$sess = retrieve_session($type, $name, 1);
my $delta = time() - $tmp;
ok ( $delta > 0, 'lock on retrieve 4' );
ok ( exists $sess->{'content'}->{'papa'}, 'lock on retrieve 5' );
ok ( exists $sess->{'content'}->{'fils'}, 'lock on retrieve 6' );
is ( $sess->{'content'}->{'papa'}, 'plop', 'lock on retrieve 7' );
is ( $sess->{'content'}->{'fils'}, 'coin', 'lock on retrieve 8' );
unlock_session($sess);
BEGIN { $nbtest += 5; }

BEGIN { 
	plan tests => $nbtest;
};
BEGIN { use_ok('FDN::Web::Session'); }

